(* Dan's interp1.ml file *)

open Ast

exception InterpUnimplemented

type heap_val = I of int | S of stmt
let mt_heap              = []
let rec get_var heap str = 
  match heap with
    [] -> I 0
  | (s,v)::tl -> if s=str then v else get_var tl str
let set_var heap str v = (str,v)::heap

let rec interp_exp h e =
  let interp_exp_r = interp_exp h in
  match e with
    Int i -> i
  | Var v -> 
      (match get_var h v with
	I i -> i
      |	S s -> 0)
  | Plus(e1,e2)  -> (interp_exp_r e1) + (interp_exp_r e2)
  | Times(e1,e2) -> (interp_exp_r e1) * (interp_exp_r e2)
 
let rec interp_step h s =
  match s with
    Skip -> (h,s)
  | Assign(v,e) -> (set_var h v (I (interp_exp h e)), Skip)
  | Seq(Skip,s2) -> (h,s2)
  | Seq(s1,s2) -> 
      let (h3,s3) = interp_step h s1 in
      (h3, Seq(s3,s2))
  | If(e,s1,s2) ->
      if((interp_exp h e) <= 0)
      then (h,s2)
      else (h,s1)
  | While(e,s1) -> (h, If(e,Seq(s1,s),Skip))
  | Ntimes(e,s) ->
      let n = interp_exp h e in
      if(n <= 0)
      then (h,Skip)
      else (h,Seq(s,Ntimes(Int(n-1),s)))
  | AssignStmt(v,s) -> (set_var h v (S s), Skip)
  | Run v ->
      (match get_var h v with
	I i -> (h,Skip)
      |	S s -> (h,s))

let rec iter h s =
  match (h,s) with
    (_,Skip) -> interp_exp h (Var "ans")
  | _ -> let (h',s') = interp_step h s in iter h' s'
  
let interp = iter mt_heap

(* the next two bindings really belong in a "main" module, but this
keeps all the code you need in one place *)

(* expect one command-line argument, a file to parse and interpret *)
(* you do not need to understand this interaction with the system *)
let get_prog () =
  let argv = Sys.argv in
  let _ = 
    if Array.length argv != 2
    then (prerr_string ("usage: " ^ argv.(0) ^ " [file-to-interpret]\n");
	  exit 1) in
  let ch = open_in argv.(1) in
  Parse.program Lex.lexer (Lexing.from_channel ch)

let _ =
  let prog = get_prog () in
  print_string ("ans = " ^ string_of_int (interp prog) ^"\n")
